#ifndef _PHM_H_
#define _PHM_H_

#include <p33FJ64MC804.h>
#include <stdio.h>
#include <stdarg.h>

typedef unsigned char		uint8_t;
typedef unsigned short int	uint16_t;
typedef unsigned long int	uint32_t;

typedef signed char		int8_t;
typedef signed short int	int16_t;
typedef signed long int		int32_t;

typedef signed char		sint8_t;

extern int	dir, qeiCnt;
extern uint32_t runTime;

enum dir_e
{
    STOP,
    FWD,
    REV
};

#define	CONFIG1			0x01
#define	  CLK_EN		(1 << 5)

#define	ADS_CMD_WAKEUP		0x02
#define	ADS_STANDBY		0x04
#define	ADS_CMD_RESET		0x06
#define	ADS_CMD_START		0x08
#define	ADS_CMD_STOP		0x0A
    
#define	ADS_CMD_RDATAC		0x10
#define	ADS_CMD_SDATAC		0x11
#define	ADS_CMD_RDATA		0x12

#define	ADS_CMD_RREG		0x20
#define	ADS_CMD_WREG		0x40

#define	GAIN1			(1 << 4)
#define	GAIN2			(2 << 4)
#define	GAIN3			(3 << 4)
#define	GAIN4			(3 << 4)
#define	GAIN6			(0 << 4)
#define	GAIN8			(5 << 4)
#define	GAIN12			(6 << 4)

/*
 * Output mapping info
 */
#define	RPO_NULL		0
#define	RPO_C1OUT		1
#define	RPO_C2OUT		2
#define	RPO_U1TX		3
#define	RPO_U1RTS		4
#define	RPO_U2TX		5
#define	RPO_U2RTS		6
#define	RPO_SDO1		7
#define	RPO_SCK1		8
#define	RPO_SS1			9
#define	RPO_SDO2		10
#define	RPO_SCK3		11
#define	RPO_SS2			12
#define	RPO_CSDO		13
#define	RPO_CSCK		14
#define	RPO_COFS		15
#define	RPO_C1TX		16
#define	RPO_OC1			18
#define	RPO_OC2			19
#define	RPO_OC3			20
#define	RPO_OC4			21
#define	RPO_OC5			22

typedef struct QUEUE_S
{
    int		qlen;
    int		bcnt;
    uint8_t	*sptr;
    uint8_t	*iptr;
    uint8_t	*eptr;
    uint8_t	*wptr;
} queue_t;

#define	LOCK_I2C	(1 << 0)

#define	ADS_DRDYn	PORTAbits.RA9

#define	ADS_START	LATBbits.LATB6
#define	ADS_RESETn	LATBbits.LATB7
#define	ADS_PWRDNn	LATBbits.LATB8

#define	ADS_SCLK	LATCbits.LATC3
#define	ADS_MOSI	LATCbits.LATC4
#define	ADS_MISO	PORTCbits.RC5

extern void	uartInit(void);
extern void	uartPutc(uint8_t ch);
extern void	uartPuts(const char *s);
extern uint8_t	uartGetc(void);
extern char	*uart_gets(char *s);
extern int	uartCharAvail(void);
//extern int	printf(const char *fmt, ...);

extern void	queueInit(queue_t *a, uint8_t *buf, int bufSize);
extern int	queueExtract(queue_t *s, uint8_t *qdata);
extern int	queueInsert(queue_t *q, uint8_t qdata);

extern void	qeiInit(void);

extern void	pwmInit(void);
extern void	pwmSet(int pwm);

extern void	tmrInit(void);
extern void	pause(uint16_t delay);

#endif
