#include "demo.h"

/*
 * Initialize a byte oriented queue. The queue consists
 * of a queue header and buffer. Pointers and counters are
 * used to improve speed.
 *
 * sptr - start pointer - start of queue data
 * wptr	- wrap pointer - end of data + 1
 * iptr - insertion pointer
 * eptr - extraction pointer
 * qlen - size of allocated data
 * bcnt - current data size
 * mcnt - max data ever used
 *
 */
void queueInit(queue_t *q, uint8_t *buf, int bufSize)
{
    q->sptr = buf;
    q->iptr = buf;
    q->eptr = buf;
    q->wptr = buf + bufSize;
    q->qlen = bufSize;
    q->bcnt = 0;
}

/*
 * Insert an 8-bit data byte into the specified queue.
 *
 * Return 0 if there is no room in the queue, return 1
 * on success. Interrupts are temporally disabled.
 */
int queueInsert(queue_t *q, uint8_t qdata)
{
    uint16_t
	sr;

    if (q->bcnt >= q->qlen) return(0);

    sr = SR;
    SR = sr | (7 << 5);

    *q->iptr++ = qdata;
    if (q->iptr == q->wptr) q->iptr = q->sptr;
    q->bcnt++;

    SR = sr;				// restore interrupts

    return(1);
}

/*
 * Extract an 8-bit data byte from the specified queue.
 *
 * Returns 0 if there is no data available, 1 otherwise.
 *
 */
int queueExtract(queue_t *q, uint8_t *qdata)
{
    uint16_t
	sr;

    if (q->bcnt == 0) return(0);

    sr = SR;
    SR = sr | (7 << 5);			// disable interrupts

    *qdata = *q->eptr;
    q->eptr++;
    if (q->eptr == q->wptr) q->eptr = q->sptr;
    q->bcnt--;

    SR = sr;				// restore interrupts
    
    return(1);
}

int queueLength(queue_t *q)
{
    return(q->bcnt);
}
